import numpy as np
import panel as pn
from bokeh.plotting import figure
from bokeh.models import ColumnDataSource

pn.extension(design="material")

# Generate random data
np.random.seed(42)
x = np.random.rand(50)
y = 2 * x + 1 + 0.1 * np.random.randn(50)

# Initial slope and intercept
initial_slope = 2
initial_intercept = 1

# Calculate initial predictions
y_pred = initial_slope * x + initial_intercept

# Create plot
plot = figure(width=800, height=400)
plot.scatter(x, y)

# Define line and vertical lines
source = ColumnDataSource(data={
    'x': x,
    'y_true': y,
    'y_pred': y_pred,
    'y_top': np.maximum(y_pred, y),
    'y_bottom': np.minimum(y_pred, y)
})
plot.line(x='x', y='y_pred', source=source, color='red', line_width=2)
plot.segment(x0='x', y0='y_true', x1='x', y1='y_pred', source=source, color='blue', line_width=1)

# Widgets
slope = pn.widgets.FloatSlider(name='Slope', start=-5, end=5, value=initial_slope)
intercept = pn.widgets.FloatSlider(name='Intercept', start=-5, end=5, value=initial_intercept)

# Residual sum of squares
rss_text = pn.widgets.StaticText(name='Residual Sum of Squares', value='0')

def update_regression(event=None):
    new_slope = slope.value
    new_intercept = intercept.value

    y_pred = new_slope * x + new_intercept
    source.data['y_pred'] = y_pred
    source.data['y_top'] = np.maximum(y_pred, y)
    source.data['y_bottom'] = np.minimum(y_pred, y)

    residuals = y - y_pred
    rss = np.sum(residuals ** 2)
    rss_text.value = f"{rss:.4f}"

slope.param.watch(update_regression, 'value')
intercept.param.watch(update_regression, 'value')

# Initial update to set correct RSS and avoid initial line glitch
update_regression()

# Layout
layout = pn.Column(
    slope,
    intercept,
    rss_text,
    pn.pane.Bokeh(plot)
)

# Serve the app
layout.servable(target="panel")
